/* netproto.c: network protocol */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "netproto.h"

#define SEND_BUFFER_SZ 65536
char send_buffer[SEND_BUFFER_SZ];
int buffer_pos = 0;

void lookup_server(char *host, int port, struct sockaddr_in *saddr)
{
  struct hostent *hp;
  hp = gethostbyname(host);
  saddr->sin_port = htons(port);
  saddr->sin_family = AF_INET;
  if (hp == NULL) saddr->sin_addr.s_addr = inet_addr(host);
  else saddr->sin_addr = *(struct in_addr *)(hp->h_addr_list[0]);
}

int connect_to_server(struct sockaddr_in *saddr)
{
  int sock, one;
  
  sock = socket(AF_INET, SOCK_STREAM, 0);
  if (sock == -1) return -1;
  if (connect(sock, saddr, sizeof *saddr) < 0) return -1;
  one = 1;
  setsockopt(sock, SOL_TCP, TCP_NODELAY, &one, sizeof(one));
  return sock;
}

int do_send_data(int sock, unsigned int len, const char *s)
{
  int val;
  
  while (len > 0) {
    val = __libc_write(sock, s, len);
    if (val == -1) return 0;
    len -= val; 
    s += val;
  }
  return 1;
}

int flush_send(int sock)
{
  int temp;
  if (buffer_pos==0) return 1;
  temp = do_send_data(sock, buffer_pos, send_buffer);
  buffer_pos = 0;
  return temp;
}

int send_data(int sock, unsigned int len, const char *s)
{
  if (len+buffer_pos < SEND_BUFFER_SZ) {
    memcpy(send_buffer+buffer_pos, s, len);
    buffer_pos += len;
    return 1;
  }
  if (!flush_send(sock)) return 0;
  return do_send_data(sock, len, s);
}

int send_int(int sock, int i)
{
  return send_data(sock, sizeof(int), (char *)&i);
}

int send_string(int sock, const char *s)
{
  if (!send_int(sock, strlen(s))) return 0;
  return send_data(sock, strlen(s), s);
}

int recv_data(int sock, unsigned int len, char *s)
{
  int val;
  while (len>0) {
    val = __libc_read(sock, s, len);
    if (val <= 0) return 0;
    len -= val;
    s += val;
  }
  return 1;
}

int recv_int(int sock, int *i)
{
  return recv_data(sock, sizeof(int), (char *)i);
}

int recv_string(int sock, char **s)
{
  int len;
  char *buf;
  
  if (!recv_int(sock, &len)) return 0;
  buf = (char *)malloc(len+1);
  if (buf == NULL) return 0;
  if (!recv_data(sock, len, buf)) { free(buf); return 0; }
  *s = buf;
  return 1;
}
